@@ -45,40 +45,68 @@
 //! # Crate Features
 //!
 //! By default, no crate features are enabled.
+//! - `std`: Implement traits from `std`, such as [`Error`] on [`ParseError`].
 //! - `macro`: Enable the [`bounded_integer!`] macro.
 //! - `types`: Enable the bounded integer types that use const generics.
-//! - `serde`: Implement `Serialize` and `Deserialize` for the bounded integers, making sure all
-//! values will never be out of bounds.
+//! - `arbitrary1`: Implement [`Arbitrary`] for the bounded integers. This is useful when using
+//! bounded integers as fuzzing inputs.
+//! - `bytemuck1`: Implement [`Contiguous`] for all bounded integers, and [`Zeroable`] for
+//! macro-generated bounded integers that support it.
+//! - `serde1`: Implement [`Serialize`] and [`Deserialize`] for the bounded integers, making sure all
+//! values will never be out of bounds. This has a deprecated alias `serde`.
+//! - `zerocopy06`: Implement [`AsBytes`] and [`Unaligned`] for macro-generated bounded integers. We
+//! can't implement them for const generic bounded integers due to [limitations in
+//! Zerocopy](https://bugs.fuchsia.dev/p/fuchsia/issues/detail?id=84475).
 //! - `step_trait`: Implement the [`Step`] trait which allows the bounded integers to be easily used
 //! in ranges. This will require you to use nightly and place `#![feature(step_trait)]` in your
 //! crate root if you use the macro.
 //!
 //! [`bounded_integer!`]: https://docs.rs/bounded-integer/*/bounded_integer/macro.bounded_integer.html
 //! [`examples`]: https://docs.rs/bounded-integer/*/bounded_integer/examples/
+//! [`Arbitrary`]: https://docs.rs/arbitrary/1/arbitrary/trait.Arbitrary.html
+//! [`Contiguous`]: https://docs.rs/bytemuck/1/bytemuck/trait.Contiguous.html
+//! [`Zeroable`]: https://docs.rs/bytemuck/1/bytemuck/trait.Zeroable.html
+//! [`Serialize`]: https://docs.rs/serde/1/serde/trait.Serialize.html
+//! [`Deserialize`]: https://docs.rs/serde/1/serde/trait.Deserialize.html
+//! [`AsBytes`]: https://docs.rs/zerocopy/0.6/zerocopy/trait.AsBytes.html
+//! [`Unaligned`]: https://docs.rs/zerocopy/0.6/zerocopy/trait.Unaligned.html
 //! [`Step`]: https://doc.rust-lang.org/nightly/core/iter/trait.Step.html
+//! [`Error`]: https://doc.rust-lang.org/stable/std/error/trait.Error.html
+//! [`ParseError`]: https://docs.rs/bounded-integer/*/bounded_integer/struct.ParseError.html
 #![cfg_attr(feature = "step_trait", feature(step_trait))]
 #![cfg_attr(doc_cfg, feature(doc_cfg))]
+#![allow(clippy::single_component_path_imports)] // https://github.com/rust-lang/rust-clippy/issues/7106
 #![no_std]
 
+#[cfg(feature = "std")]
+extern crate std;
+
 #[cfg(feature = "types")]
 mod types;
 #[cfg(feature = "types")]
 pub use types::*;
 
+mod parse;
+pub use parse::{ParseError, ParseErrorKind};
+
 #[doc(hidden)]
 #[cfg(feature = "macro")]
 pub mod __private {
-    #[cfg(feature = "serde")]
-    pub use ::serde;
+    #[cfg(feature = "arbitrary1")]
+    pub use ::arbitrary1;
 
-    #[cfg(all(not(feature = "serde"), not(feature = "step_trait")))]
-    pub use bounded_integer_macro::not_serde_not_step_trait as proc_macro;
-    #[cfg(all(not(feature = "serde"), feature = "step_trait"))]
-    pub use bounded_integer_macro::not_serde_step_trait as proc_macro;
-    #[cfg(all(feature = "serde", not(feature = "step_trait")))]
-    pub use bounded_integer_macro::serde_not_step_trait as proc_macro;
-    #[cfg(all(feature = "serde", feature = "step_trait"))]
-    pub use bounded_integer_macro::serde_step_trait as proc_macro;
+    #[cfg(feature = "bytemuck1")]
+    pub use ::bytemuck1;
+
+    #[cfg(feature = "serde1")]
+    pub use ::serde1;
+
+    #[cfg(feature = "zerocopy06")]
+    pub use ::zerocopy06;
+
+    pub use bounded_integer_macro::bounded_integer as proc_macro;
+
+    pub use crate::parse::{error_above_max, error_below_min, FromStrRadix};
 }
 
 #[cfg(feature = "__examples")]
@@ -166,7 +194,69 @@ pub mod examples;
 #[cfg_attr(doc_cfg, doc(cfg(feature = "macro")))]
 #[macro_export]
 macro_rules! bounded_integer {
-    ($($tt:tt)*) => {
-        $crate::__private::proc_macro!([$crate] $($tt)*);
+    ($($tt:tt)*) => { $crate::__bounded_integer_inner! { $($tt)* } };
+}
+
+// `bounded_integer!` needs to generate different output depending on what feature flags are
+// enabled in this crate. We can't propagate feature flags from this crate directly to
+// `bounded-integer-macro` because it is an optional dependency, so we instead dynamically pass
+// options into the macro depending on which feature flags are enabled here.
+
+#[cfg(feature = "macro")]
+block! {
+    let arbitrary1: ident = cfg_bool!(feature = "arbitrary1");
+    let bytemuck1: ident = cfg_bool!(feature = "bytemuck1");
+    let serde1: ident = cfg_bool!(feature = "serde1");
+    let zerocopy06: ident = cfg_bool!(feature = "zerocopy06");
+    let step_trait: ident = cfg_bool!(feature = "step_trait");
+    let d: tt = dollar!();
+
+    #[doc(hidden)]
+    #[macro_export]
+    macro_rules! __bounded_integer_inner2 {
+        ($d($d tt:tt)*) => {
+            $crate::__private::proc_macro! {
+                [$crate] $arbitrary1 $bytemuck1 $serde1 $zerocopy06 $step_trait $d($d tt)*
+            }
+        };
+    }
+
+    // Workaround for `macro_expanded_macro_exports_accessed_by_absolute_paths`
+    #[doc(hidden)]
+    pub use __bounded_integer_inner2 as __bounded_integer_inner;
+}
+
+#[cfg(feature = "macro")]
+macro_rules! cfg_bool {
+    ($meta:meta) => {
+        #[cfg($meta)]
+        ret! { true }
+        #[cfg(not($meta))]
+        ret! { false }
     };
 }
+#[cfg(feature = "macro")]
+use cfg_bool;
+
+#[cfg(feature = "macro")]
+macro_rules! dollar {
+    () => { ret! { $ } };
+}
+#[cfg(feature = "macro")]
+use dollar;
+
+#[cfg(feature = "macro")]
+macro_rules! block {
+    { let $ident:ident: $ty:ident = $macro:ident!($($macro_args:tt)*); $($rest:tt)* } => {
+        macro_rules! ret {
+            ($d:tt) => {
+                macro_rules! ret { ($d $ident: $ty) => { block! { $($rest)* } } }
+                $macro! { $($macro_args)* }
+            }
+        }
+        dollar! {}
+    };
+    { $($rest:tt)* } => { $($rest)* };
+}
+#[cfg(feature = "macro")]
+use block;
